@@ -19,6 +19,7 @@
 #include <gensio/gensio_class.h>
 
 #include "utils.h"
+#include "gensio_net.h"
 
 static unsigned int gensio_log_mask =
     (1 << GENSIO_LOG_FATAL) | (1 << GENSIO_LOG_ERR);
@@ -1277,8 +1278,11 @@ str_to_gensio_accepter(const char *str,
 
 	str += len;
 	err = gensio_scan_args(o, &str, NULL, &args);
-	if (!err)
+	if (!err) {
+	    while (isspace(*str))
+		str++;
 	    err = r->handler(str, args, o, cb, user_data, accepter);
+	}
 	if (args)
 	    gensio_argv_free(o, args);
 	return err;
@@ -1488,8 +1492,11 @@ str_to_gensio(const char *str,
 
 	str += len;
 	err = gensio_scan_args(o, &str, NULL, &args);
-	if (!err)
+	if (!err) {
+	    while (isspace(*str))
+		str++;
 	    err = r->handler(str, args, o, cb, user_data, gensio);
+	}
 	if (args)
 	    gensio_argv_free(o, args);
 	return err;
@@ -1946,7 +1953,12 @@ struct gensio_def_entry builtin_defaults[] = {
     /* Defaults for TCP, UDP, and SCTP. */
     { "nodelay",	GENSIO_DEFAULT_BOOL,	.def.intval = 0 },
     { "laddr",		GENSIO_DEFAULT_STR,	.def.strval = NULL },
-    /* sctp */
+    /* TCP only */
+#ifdef HAVE_TCPD_H
+    { "tcpd",		GENSIO_DEFAULT_ENUM,	.enums = tcpd_enums,
+						.def.intval = GENSIO_TCPD_ON },
+#endif
+    /* SCTP only */
     { "instreams",	GENSIO_DEFAULT_INT,	.min = 1, .max = INT_MAX,
 						.def.intval = 1 },
     { "ostreams",	GENSIO_DEFAULT_INT,	.min = 1, .max = INT_MAX,
@@ -2035,6 +2047,8 @@ gensio_default_init(void *cb_data)
 						  NULL, 0);
 }
 
+extern void gensio_sol_cleanup_mem(void);
+
 void
 gensio_cleanup_mem(struct gensio_os_funcs *o)
 {
@@ -2077,6 +2091,7 @@ gensio_cleanup_mem(struct gensio_os_funcs *o)
 
     memset(&gensio_default_initialized, 0, sizeof(gensio_default_initialized));
     memset(&gensio_base_initialized, 0, sizeof(gensio_base_initialized));
+    gensio_sol_cleanup_mem();
 }
 
 static void
