@@ -32,6 +32,8 @@
 #include <gensio/gensio_osops.h>
 #include <gensio/gensio_builtins.h>
 
+#include "gensio_net.h"
+
 struct net_data {
     struct gensio_os_funcs *o;
 
@@ -331,6 +333,12 @@ net_gensio_alloc(const struct gensio_addr *iai, const char * const args[],
 	return err;
     }
 
+    err = gensio_get_default(o, type, "nodelay", false,
+			     GENSIO_DEFAULT_BOOL, NULL, &ival);
+    if (err)
+	return err;
+    nodelay = ival;
+
     for (i = 0; args && args[i]; i++) {
 	if (gensio_check_keyds(args[i], "readbuf", &max_read_size) > 0)
 	    continue;
@@ -485,7 +493,10 @@ struct netna_data {
     char *group;
 #endif
 
+#ifdef HAVE_TCPD_H
+    enum gensio_tcpd_options tcpd;
     char *tcpd_progname;
+#endif
 
     unsigned int   nr_acceptfds;
     unsigned int   nr_accept_close_waiting;
@@ -574,14 +585,23 @@ netna_readhandler(int fd, void *cbdata)
 	return;
     }
 
-    if (nadata->istcp) {
-	if (gensio_os_check_tcpd_ok(new_fd, nadata->tcpd_progname)) {
+#ifdef HAVE_TCPD_H
+    if (nadata->istcp && nadata->tcpd != GENSIO_TCPD_OFF) {
+	const char *msg = gensio_os_check_tcpd_ok(new_fd,
+						  nadata->tcpd_progname);
+
+	if (msg) {
+	    if (nadata->tcpd == GENSIO_TCPD_PRINT) {
+		struct gensio_sg sg[1] = { { msg, strlen(msg) } };
+		gensio_os_send(nadata->o, new_fd, sg, 1, NULL, 0);
+	    }
 	    gensio_acc_log(nadata->acc, GENSIO_LOG_INFO,
 			   "Error accepting net gensio: tcpd check failed");
 	    err = GE_INVAL;
 	    goto out_err;
 	}
     }
+#endif
 
     tdata = nadata->o->zalloc(nadata->o, sizeof(*tdata));
     if (!tdata) {
@@ -644,8 +664,11 @@ netna_readhandler(int fd, void *cbdata)
 	    if (new_fd != -1)
 		gensio_os_close(nadata->o, &new_fd);
 	}
-    } else if (raddr) {
-	gensio_addr_free(raddr);
+    } else {
+	if (raddr)
+	    gensio_addr_free(raddr);
+	if (new_fd != -1)
+	    gensio_os_close(nadata->o, &new_fd);
     }
 }
 
@@ -1031,6 +1054,15 @@ netna_base_acc_op(struct gensio_accepter *acc, int op,
     }
 }
 
+#ifdef HAVE_TCPD_H
+struct gensio_enum_val tcpd_enums[] = {
+    { "on",	GENSIO_TCPD_ON },
+    { "print",	GENSIO_TCPD_PRINT },
+    { "off", 	GENSIO_TCPD_OFF },
+    { NULL }
+};
+#endif
+
 static int
 net_gensio_accepter_alloc(struct gensio_addr *iai,
 			  const char * const args[],
@@ -1051,6 +1083,7 @@ net_gensio_accepter_alloc(struct gensio_addr *iai,
 #endif
 #ifdef HAVE_TCPD_H
     const char *tcpdname = NULL;
+    enum gensio_tcpd_options tcpd = GENSIO_TCPD_ON;
 #endif
     unsigned int i;
     int err, ival;
@@ -1068,6 +1101,14 @@ net_gensio_accepter_alloc(struct gensio_addr *iai,
     }
     reuseaddr = ival;
 
+#ifdef HAVE_TCPD_H
+    err = gensio_get_default(o, type, "tcpd", false,
+			     GENSIO_DEFAULT_INT, NULL, &ival);
+    if (err)
+	return err;
+    tcpd = ival;
+#endif
+
     for (i = 0; args && args[i]; i++) {
 	if (gensio_check_keyds(args[i], "readbuf", &max_read_size) > 0)
 	    continue;
@@ -1082,6 +1123,11 @@ net_gensio_accepter_alloc(struct gensio_addr *iai,
 #ifdef HAVE_TCPD_H
 	if (istcp && gensio_check_keyvalue(args[i], "tcpdname", &tcpdname))
 	    continue;
+	if (istcp && gensio_check_keyenum(args[i], "tcpd",
+					  tcpd_enums, &ival) > 0) {
+	    tcpd = ival;
+	    continue;
+	}
 #endif
 #if HAVE_UNIX
 	if (!istcp && gensio_check_keymode(args[i], "umode", &umode) > 0) {
@@ -1135,6 +1181,7 @@ net_gensio_accepter_alloc(struct gensio_addr *iai,
 #endif
 
 #ifdef HAVE_TCPD_H
+    nadata->tcpd = tcpd;
     if (tcpdname)
 	nadata->tcpd_progname = strdup(tcpdname);
 #endif
