@@ -700,6 +700,15 @@ udpn_start_close(struct udpn_data *ndata,
     struct udpna_data *nadata = ndata->nadata;
 
     if (nadata->pending_data_owner == ndata) {
+	if (ndata->deferred_read) {
+	    /*
+	     * If there is a read pending on a deferred op, it won't
+	     * get called now that this is closed.  So cancel it so
+	     * the close will finish
+	     */
+	    ndata->deferred_read = false;
+	    ndata->in_read = false;
+	}
 	nadata->pending_data_owner = NULL;
 	nadata->data_pending_len = 0;
     }
@@ -812,7 +821,7 @@ udpn_set_read_callback_enable(struct gensio *io, bool enabled)
 	/* Call the read from the selector to avoid lock nesting issues. */
 	udpna_start_deferred_op(nadata);
     } else {
-	udpna_check_read_state(ndata->nadata);
+	udpna_check_read_state(nadata);
     }
  out_unlock:
     udpna_unlock(nadata);
