@@ -979,7 +979,7 @@ static void
 serialdev_timeout(struct gensio_timer *t, void *cb_data)
 {
     struct sterm_data *sdata = cb_data;
-    int val;
+    int val, rv;
     unsigned int modemstate = 0;
     bool force_send;
 
@@ -991,8 +991,9 @@ serialdev_timeout(struct gensio_timer *t, void *cb_data)
     sdata->handling_modemstate = true;
     sterm_unlock(sdata);
 
-    if (ioctl(sdata->fd, TIOCMGET, &val) != 0)
-	return;
+    rv = ioctl(sdata->fd, TIOCMGET, &val);
+    if (rv)
+	goto out_restart;
 
     if (val & TIOCM_CD)
 	modemstate |= SERGENSIO_MODEMSTATE_CD;
@@ -1007,7 +1008,6 @@ serialdev_timeout(struct gensio_timer *t, void *cb_data)
     /* Bits for things that changed. */
     modemstate |= (modemstate ^ sdata->last_modemstate) >> 4;
     sdata->last_modemstate = modemstate & sdata->modemstate_mask;
-    modemstate &= sdata->last_modemstate;
     force_send = !sdata->sent_first_modemstate;
     sdata->sent_first_modemstate = true;
     sterm_unlock(sdata);
@@ -1024,6 +1024,7 @@ serialdev_timeout(struct gensio_timer *t, void *cb_data)
 		  (unsigned char *) &modemstate, &vlen, NULL);
     }
 
+ out_restart:
     if (sdata->modemstate_mask) {
 	gensio_time timeout = {1, 0};
 
@@ -1039,17 +1040,16 @@ static int
 sterm_modemstate(struct sergensio *sio, unsigned int val)
 {
     struct sterm_data *sdata = sergensio_get_gensio_data(sio);
+    gensio_time timeout = {0, 0};
 
     sterm_lock(sdata);
     sdata->modemstate_mask = val;
+    sdata->sent_first_modemstate = false;
     sterm_unlock(sdata);
-    if (sdata->modemstate_mask) {
-	gensio_time timeout = {0, 1000};
 
-	sdata->o->start_timer(sdata->timer, &timeout);
-    } else {
-	sdata->o->stop_timer(sdata->timer);
-    }
+    /* Cause an immediate send of the modemstate. */
+    sdata->o->stop_timer(sdata->timer);
+    sdata->o->start_timer(sdata->timer, &timeout);
     return 0;
 }
 
@@ -1547,7 +1547,8 @@ handle_speedstr(g_termios *termio, const char *str, bool custspeed)
     const char *rest = "";
 
     val = speedstr_to_speed(str, &rest);
-    if (val == -1)
+    if (val < 10)
+	/* Some parameters start a digit, ignore them. */
 	return GE_INVAL;
     if (set_termios_from_speed(termio, val, rest, custspeed) == -1)
 	return GE_INVAL;
