@@ -14,8 +14,6 @@ check_pipe_dev()
 isim = ipmisimdaemon.IPMISimDaemon(o, ttypipe[1])
 io1 = alloc_io(o, "serialdev," + ttypipe[0] + ",115200")
 io2 = alloc_io(o, "ipmisol,lan -U ipmiusr -P test -p 9001 localhost,115200")
-rb = os.urandom(104857)
-test_dataxfer(io1, io2, rb, timeout=20000)
+do_medium_test(io1, io2, timeout=30000)
 io_close(io1)
 io_close(io2)
-print("  Success!")
