@@ -14,7 +14,6 @@ check_pipe_dev()
 isim = ipmisimdaemon.IPMISimDaemon(o, ttypipe[1])
 io1 = alloc_io(o, "serialdev," + ttypipe[0] + ",9600")
 io2 = alloc_io(o, "ipmisol,lan -U ipmiusr -P test -p 9001 localhost,9600")
-test_dataxfer(io1, io2, "This is a test string!")
+do_small_test(io1, io2)
 io_close(io1)
 io_close(io2)
-print("  Success!")
